@@ -1,14 +1,21 @@
 # coding: utf-8
-"""Classes to handle recipe inputs and outputs."""
+"""Class to handle recipe inputs and outputs."""
 from __future__ import division
 
 import os
 import json
 import importlib
 import shutil
+import subprocess
 
+from ladybug.futil import preparedir, nukedir, copy_file_tree
 from honeybee.config import folders
-from ladybug.futil import preparedir, copy_file_tree
+from honeybee_radiance.config import folders as rad_folders
+from honeybee.model import Model
+
+from .settings import RecipeSettings
+from .version import check_radiance_date, check_openstudio_version, \
+    check_energyplus_version
 
 
 class Recipe(object):
@@ -34,13 +41,14 @@ class Recipe(object):
         * inputs
         * outputs
     """
+    MODEL_EXTENSIONS = ('.hbjson', '.hbpkl', '.dfjson', '.dfpkl', '.json', '.pkl')
 
     def __init__(self, recipe_name):
         # check to be sure that the requested recipe is installed
         install_folder = os.path.dirname(__file__)
-        recipe_folder = os.path.join(install_folder, recipe_name)
+        recipe_folder = os.path.join(install_folder, recipe_name.replace('-', '_'))
         if os.path.isdir(recipe_folder):  # it's a recipe in this package
-            self._name = recipe_name
+            self._name = recipe_name.replace('-', '_')
             self._path = recipe_folder
         elif os.path.isdir(recipe_name):  # it's an externally-installed recipe
             self._name = os.path.basename(recipe_name)
@@ -88,8 +96,20 @@ class Recipe(object):
         If unset, this will be a folder called unnamed_project within the user's
         default simulation folder
         """
-        return self._default_project_folder if self._default_project_folder is not None \
-            else os.path.join(folders.default_simulation_folder, 'unnamed_project')
+        if self._default_project_folder is not None:
+            return self._default_project_folder
+        else:
+            def_sim = folders.default_simulation_folder
+            for inp in self._inputs:
+                if inp.name == 'model':
+                    if isinstance(inp.value, Model):
+                        return os.path.join(def_sim, inp.value.identifier)
+                    elif isinstance(inp.value, str) and os.path.isfile(str(inp.value)):
+                        model = os.path.basename(inp.value)
+                        for ext in self.MODEL_EXTENSIONS:
+                            model = model.replace(ext, '')
+                        return os.path.join(def_sim, model)
+            return os.path.join(def_sim, 'unnamed_project')
 
     @default_project_folder.setter
     def default_project_folder(self, path):
@@ -117,9 +137,19 @@ class Recipe(object):
         """Get a tuple of RecipeOutput objects for the recipe's outputs."""
         return tuple(self._outputs)
 
+    @property
+    def input_names(self):
+        """Get a tuple of text for the recipe's input names."""
+        return tuple(inp.name for inp in self._inputs)
+
+    @property
+    def output_names(self):
+        """Get a tuple of text for the recipe's output names."""
+        return tuple(otp.name for otp in self._outputs)
+
     def input_value_by_name(self, input_name, input_value):
         """Set the value of an input given the input name.
-        
+
         Args:
             input_name: Text for the name of the input to be set. For example,
                 'radiance-parameters'.
@@ -161,14 +191,19 @@ class Recipe(object):
         inp_dict = {}
         for inp in self.inputs:
             inp.handle_value()
-            if inp.is_path:  # we need to copy the path to the project folder
+            if inp.is_path and inp.value is not None:  # copy artifact to project folder
                 path_basename = os.path.basename(inp.value)
-                dest = os.path.join(project_folder, path_basename)
+                dest = os.path.join(p_fold, path_basename)
                 if os.path.isfile(inp.value):
-                    shutil.copyfile(inp.value, dest)
+                    try:
+                        shutil.copyfile(inp.value, dest)
+                    except shutil.SameFileError:
+                        pass  # the file is already in the right place; no need to copy
                 elif os.path.isdir(inp.value):
                     copy_file_tree(inp.value, dest, overwrite=True)
                 inp_dict[inp.name] = path_basename
+            elif inp.is_path and inp.value is None:  # conditional artifact; ignore it
+                pass
             else:
                 inp_dict[inp.name] = inp.value
         # write the inputs dictionary to a file
@@ -176,11 +211,100 @@ class Recipe(object):
             json.dump(inp_dict, fp, indent=indent)
         return file_path
 
+    def run(self, settings=None, radiance_check=False, openstudio_check=False,
+            energyplus_check=False, queenbee_path=None, silent=False):
+        """Run the recipe using the queenbee local run command.
+
+        Args:
+            settings: An optional RecipeSettings object or RecipeSettings string
+                to dictate the settings of the recipe run (eg. the number of
+                workers or the project folder). If None, default settings will
+                be assumed. (Default: None).
+            radiance_check: Boolean to note whether the installed version of
+                Radiance should be checked before executing the recipe. If there
+                is no compatible version installed, an exception will be raised
+                with a clear error message. (Default: False).
+            openstudio_check: Boolean to note whether the installed version of
+                OpenStudio should be checked before executing the recipe. If there
+                is no compatible version installed, an exception will be raised
+                with a clear error message. (Default: False).
+            energyplus_check: Boolean to note whether the installed version of
+                EnergyPlus should be checked before executing the recipe. If there
+                is no compatible version installed, an exception will be raised
+                with a clear error message. (Default: False).
+            queenbee_path: Optional path to the queenbee executable. If None, the
+                queenbee within the ladybug_tools Python folder will be used.
+                Setting this to just 'queenbee' will use the system Python.
+            silent: Boolean to note whether the recipe should be run silently on
+                Windows (True) or with a command window (False). (Default: False).
+
+        Returns:
+            Path to the project folder containing the recipe results.
+        """
+        # perform any simulation engine checks
+        if radiance_check:
+            check_radiance_date()
+        if openstudio_check:
+            check_openstudio_version()
+        if energyplus_check:
+            check_energyplus_version()
+
+        # parse the settings or use default ones
+        if settings is not None:
+            settings = RecipeSettings.from_string(settings) \
+                if isinstance(settings, str) else settings
+        else:
+            settings = RecipeSettings()
+
+        # get the folder out of which the recipe will be executed
+        folder = self.default_project_folder if settings.folder is None \
+            else settings.folder
+        if not os.path.isdir(folder):
+            preparedir(folder)  # create the directory if it's not there
+
+        # delete any existing result files unless reload_old is True
+        if not settings.reload_old and self.simulation_id is not None:
+            wf_folder = os.path.join(folder, self.simulation_id)
+            if os.path.isdir(wf_folder):
+                nukedir(wf_folder, rmdir=True)
+
+        # write the inputs JSON for the recipe and set up the environment variables
+        inputs_json = self.write_inputs_json(folder)
+        genv = {}
+        genv['PATH'] = rad_folders.radbin_path
+        genv['RAYPATH'] = rad_folders.radlib_path
+        env_args = ['--env {}="{}"'.format(k, v) for k, v in genv.items()]
+
+        # create command
+        qb_path = os.path.join(folders.python_scripts_path, 'queenbee') \
+            if queenbee_path is None else queenbee_path
+        command = '"{qb_path}" local run "{recipe_folder}" ' \
+            '"{project_folder}" -i "{user_inputs}" --workers {workers} ' \
+            '{environment} --name {simulation_name}'.format(
+                qb_path=qb_path, recipe_folder=self.path, project_folder=folder,
+                user_inputs=inputs_json, workers=settings.workers,
+                environment=' '.join(env_args),
+                simulation_name=self.simulation_id
+            )
+
+        # execute command
+        shell = False if os.name == 'nt' and not silent else True
+        if settings.report_out:
+            process = subprocess.Popen(
+                command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=shell)
+            result = process.communicate()
+            print(result[0])
+            print(result[1])
+        else:
+            process = subprocess.Popen(command, shell=shell)
+            result = process.communicate()  # freeze the canvas while running
+        return folder
+
     def output_value_by_name(self, output_name, project_folder=None):
         """Set the value of an input given the input name.
-        
+
         Args:
-            input_name: Text for the name of the input to be set. For example, 'results'.
+            output_name: Text for the name of the output to be obtained.
             project_folder: The full path to the project folder containing
                 completed recipe results. If None, the default_project_folder on
                 this recipe will be assumed. (Default: None).
@@ -195,6 +319,41 @@ class Recipe(object):
                 'Output "{}" was not found for recipe "{}".'.format(
                     output_name, self.name))
 
+    def luigi_execution_summary(self, project_folder=None):
+        """Get a string of the luigi execution summary after the recipe has run.
+
+        Args:
+            project_folder: The full path to the project folder containing
+                completed recipe logs. If None, the default_project_folder on
+                this recipe will be assumed. (Default: None).
+        """
+        # determine the log file path from the project folder
+        proj = self.default_project_folder if project_folder is None else project_folder
+        sim_path = os.path.join(proj, self.simulation_id)
+        log_file = os.path.join(sim_path, '__logs__', 'logs.log')
+        # open the file and load the summary
+        read_lines, summary_lines = False, []
+        with open(log_file) as lf:
+            for line in lf:
+                if line.strip() == '===== Luigi Execution Summary =====':
+                    read_lines = not read_lines
+                    continue
+                elif read_lines:
+                    summary_lines.append(line)
+        return ''.join(summary_lines)
+
+    def failure_message(self, project_folder=None):
+        """Get a string of a recipe failure message that gives a summary of failed tasks.
+
+        Args:
+            project_folder: The full path to the project folder containing
+                completed recipe logs. If None, the default_project_folder on
+                this recipe will be assumed. (Default: None).
+        """
+        st_msg = 'The recipe failed to run with the following summary:\n'
+        end_msg = 'Use the report_out attribute of recipe settings to see a full report.'
+        return ''.join([st_msg, self.luigi_execution_summary(project_folder), end_msg])
+
     def ToString(self):
         return self.__repr__()
 
@@ -376,12 +535,12 @@ class RecipeOutput(_RecipeParameter):
         self._type = output_dict['from']['type']
         try:
             self._path = output_dict['from']['path']
-        except:  # not a file or a folder; this type of output is not yet supported
+        except Exception:  # not a file or a folder; type of output not yet supported
             self._path = None
 
     def value(self, simulation_folder):
         """Get the value of this output given the path to a simulation folder.
-        
+
         Args:
             simulation_folder: The path to a simulation folder that has finished
                 running. This is the path of a project folder joined with
